# -*- coding: utf-8 -*-
#
# Copyright 2012-2014 Romain Dorgueil
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from rdc.etl.util import slughifi


class UtilSlughifiTestCase(unittest.TestCase):
    def test_basic(self):
        self.assertEqual(slughifi('Test without unicode'), 'test-without-unicode')
        self.assertEqual(slughifi(u'Du français éèù!'), 'du-francais-eeu-')
        self.assertEqual(slughifi(u'Du français éèù!', strip=True), 'du-francais-eeu')


if __name__ == '__main__':
    unittest.main()
